@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2021 Zabbix SIA
+** Copyright (C) 2001-2022 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -58,11 +58,15 @@ class CTabFilter extends CBaseComponent {
 
 		for (const template of this._target.querySelectorAll('[type="text/x-jquery-tmpl"][data-template]')) {
 			this._templates[template.getAttribute('data-template')] = template;
-		};
+		}
 
 		for (const title of this._target.querySelectorAll('nav [data-target]')) {
 			item = this.create(title, options.data[index] || {});
 
+			if (index > 0) {
+				item.initUnsavedIndicator();
+			}
+
 			if (options.selected == index) {
 				item.renderContentTemplate();
 				this.setSelectedItem(item);
@@ -77,7 +81,7 @@ class CTabFilter extends CBaseComponent {
 	}
 
 	/**
-	 * Ensures item label is visible in tab filter navgation.
+	 * Ensures item label is visible in tab filter navigation.
 	 *
 	 * @param {CTabfilterItem} item    Filter item object.
 	 */
@@ -238,7 +242,7 @@ class CTabFilter extends CBaseComponent {
 	 * @return {Promise}
 	 */
 	profileUpdate(property, body) {
-		let url = new Curl('zabbix.php', false),
+		let url = new Curl('zabbix.php'),
 			signal = null;
 
 		url.setArgument('action', 'tabfilter.profile.update');
@@ -279,7 +283,6 @@ class CTabFilter extends CBaseComponent {
 
 			if (item) {
 				item.updateCounter(value);
-				return;
 			}
 		});
 
@@ -295,6 +298,7 @@ class CTabFilter extends CBaseComponent {
 	 */
 	setSelectedItem(item) {
 		this._active_item = item;
+		this._active_item.unsetExpandedSubfilters();
 		item.setSelected();
 
 		if (item !== this._timeselector) {
@@ -329,6 +333,42 @@ class CTabFilter extends CBaseComponent {
 		return null;
 	}
 
+	/**
+	 * Enable subfilter option by key and value.
+	 */
+	setSubfilter(key, value) {
+		this._active_item.setSubfilter(key, value);
+		this._active_item.updateUnsavedState();
+		this._active_item.updateApplyUrl();
+		this._active_item.setBrowserLocationToApplyUrl();
+	}
+
+	/**
+	 * Disable subfilter option by key and value.
+	 */
+	unsetSubfilter(key, value) {
+		this._active_item.unsetSubfilter(key, value);
+		this._active_item.updateUnsavedState();
+		this._active_item.updateApplyUrl();
+		this._active_item.setBrowserLocationToApplyUrl();
+	}
+
+	/**
+	 * Set expanded subfilter name.
+	 */
+	setExpandedSubfilters(name) {
+		return this._active_item.setExpandedSubfilters(name);
+	}
+
+	/**
+	 * Retrieve expanded subfilter names.
+	 *
+	 * @returns {array}
+	 */
+	getExpandedSubfilters() {
+		return this._active_item.getExpandedSubfilters();
+	}
+
 	/**
 	 * Register tab filter events, called once during initialization.
 	 */
@@ -380,6 +420,11 @@ class CTabFilter extends CBaseComponent {
 				else {
 					item.fire(TABFILTERITEM_EVENT_COLLAPSE);
 				}
+
+				if (this._timeselector instanceof CTabFilterItem && item !== this._timeselector
+						&& this._timeselector._expanded) {
+					this._timeselector.removeExpanded();
+				}
 			},
 
 			expand: (ev) => {
@@ -393,7 +438,9 @@ class CTabFilter extends CBaseComponent {
 				}
 
 				item.setExpanded();
-				this._target.querySelector('.tabfilter-content-container').classList.remove('display-none');
+
+				const tabfilter = this._target.querySelector('.tabfilter-content-container');
+				tabfilter.classList.remove('tabfilter-collapsed', 'display-none');
 			},
 
 			/**
@@ -407,7 +454,13 @@ class CTabFilter extends CBaseComponent {
 				}
 
 				item.removeExpanded();
-				this._target.querySelector('.tabfilter-content-container').classList.add('display-none');
+				const tabfilter = this._target.querySelector('.tabfilter-content-container');
+				if (tabfilter.querySelector('.tabfilter-subfilter')) {
+					tabfilter.classList.add('tabfilter-collapsed');
+				}
+				else {
+					tabfilter.classList.add('tabfilter-collapsed', 'display-none');
+				}
 			},
 
 			/**
@@ -443,9 +496,11 @@ class CTabFilter extends CBaseComponent {
 
 			/**
 			 * Event handler for 'Delete' button.
+			 *
+			 * @param {object} ev.detail.idx2  Index of deleted tab.
 			 */
-			deleteActiveFilterTab: (ev) => {
-				this.delete(this._active_item);
+			deleteFilterTab: (ev) => {
+				this.delete(this._items[ev.detail.idx2]);
 			},
 
 			/**
@@ -593,6 +648,8 @@ class CTabFilter extends CBaseComponent {
 			 * Action on 'Apply' button press.
 			 */
 			buttonApplyAction: () => {
+				this._active_item.unsetExpandedSubfilters();
+				this._active_item.emptySubfilter();
 				this._active_item.updateUnsavedState();
 				this._active_item.updateApplyUrl();
 				this._active_item.setBrowserLocationToApplyUrl();
@@ -675,7 +732,7 @@ class CTabFilter extends CBaseComponent {
 				const $item = ui.item;
 
 				/**
-				 * Remove inline style positon, left and top that stay after sortable.
+				 * Remove inline style position, left and top that stay after sortable.
 				 * This styles broken tabs layout.
 				 */
 				if ($item.css('position') === 'relative') {
@@ -709,14 +766,21 @@ class CTabFilter extends CBaseComponent {
 		this._filters_footer.querySelector('[name="filter_new"]')
 			.addEventListener('click', this._events.buttonSaveAsAction);
 		this._filters_footer.querySelector('[name="filter_apply"]')
-			.addEventListener('click', this._events.buttonApplyAction);
+			.addEventListener('click', () => {
+				if (this._active_item._index == 0) {
+					this._events.buttonUpdateAction();
+				}
+				else {
+					this._events.buttonApplyAction();
+				}
+			});
 		this._filters_footer.querySelector('[name="filter_reset"]')
 			.addEventListener('click', this._events.buttonResetAction);
 		this._filters_footer.addEventListener('click', this._events.buttonActionNotify);
 
 		this.on('keydown', this._events.keydown);
 		this.on(TABFILTERITEM_EVENT_UPDATE, this._events.updateActiveFilterTab);
-		this.on(TABFILTERITEM_EVENT_DELETE, this._events.deleteActiveFilterTab);
+		this.on(TABFILTERITEM_EVENT_DELETE, this._events.deleteFilterTab);
 		this.on('submit', (ev) => {
 			ev.preventDefault();
 			this._filters_footer.querySelector('[name="filter_apply"]').dispatchEvent(new CustomEvent('click'));
